@@ -17,6 +17,8 @@ from queue import deque
 from threading import Thread
 from typing import Any
 from typing import Callable
+from typing import Generator
+from typing import List
 from typing import Union
 
 import requests
@@ -359,40 +361,24 @@ class Action:
         """获取群组列表"""
         return self.baseSender('POST', 'GetGroupList', {"NextToken": ""}, timeout, **kwargs)
 
-    def get_group_admin_list(self, groupid: int, timeout=5, **kwargs) -> dict:
+    def get_group_admin_list(self, groupid: int, timeout=5, **kwargs) -> List[dict]:
         """获取群管理员列表"""
-        data = self.baseSender('POST', 'GetGroupUserList', {"GroupUin": groupid, "LastUin": 0}, timeout, **kwargs)
-        LastUin = data['LastUin']
-        MemberList = data['MemberList']
-        AdminList = []
-        while LastUin != 0:
-            time.sleep(1.1)
-            data = self.baseSender('POST', 'GetGroupUserList', {"GroupUin": groupid, "LastUin": LastUin}, timeout, **kwargs)
-            LastUin = data['LastUin']
-            MemberList += data['MemberList']
-            AdminList = [i for i in MemberList if i['GroupAdmin'] != 0]
-            if len(AdminList) == 10:
-                break
-        del LastUin, MemberList
-        return AdminList
+        members = self.get_group_user_list(groupid, timeout, **kwargs)
+        return [member for member in members if member['GroupAdmin'] == 1]
 
-    def get_group_user_list(self, groupid: int, timeout=5, **kwargs) -> dict:
+    def get_group_user_list(self, groupid: int, timeout=5, **kwargs) -> Generator[dict, None, None]:
         """获取群成员列表"""
-        data = self.baseSender('POST', 'GetGroupUserList', {"GroupUin": groupid, "LastUin": 0}, timeout, **kwargs)
-        #Count = data['Count']
-        #GroupUin = data['GroupUin']
-        LastUin = data['LastUin']
-        MemberList = data['MemberList']
-        while LastUin != 0:
-            time.sleep(1.1)
+        LastUin = 0
+        while True:
             data = self.baseSender('POST', 'GetGroupUserList', {"GroupUin": groupid, "LastUin": LastUin}, timeout, **kwargs)
-            #Count += data['Count']
-            #GroupUin = data['GroupUin']
-            LastUin = data['LastUin']
-            MemberList += data['MemberList']
-        return MemberList
+            LastUin = data['LastUin']  # 上面请求失败会返回空字典。但是这里不处理错误, 必须正常抛出
+            for member in data['MemberList']:
+                yield member
+            if LastUin == 0:
+                break
+            time.sleep(0.8)
 
-    def set_unique_title(self, groupid: int, userid: int, Title: str, timeout=1, **kwargs) -> dict:
+    def set_unique_title(self, groupid: int, userid: int, Title: str, timeout=5, **kwargs) -> dict:
         """设置群成员头衔"""
         return self.baseSender('POST', 'OidbSvc.0x8fc_2', {"GroupID": groupid, "UserID": userid, "NewTitle": Title}, timeout, **kwargs)
 
@@ -421,6 +407,34 @@ class Action:
             pass
         return False
 
+    def get_balance(self) -> dict:
+        '''获取QQ钱包余额'''
+    #     return self.baseSender('GET', 'GetBalance', timeout=timeout, **kwargs)
+
+    def get_status(self, timeout=20) -> dict:
+        '''获取机器人状态'''
+        rep = requests.get('{self.__host}:{self.__port}/v1/ClusterInfo', timeout=timeout)
+        return rep.json()
+
+    def send_single_red_bag(self) -> dict:
+        '''发送群/好友红包'''
+        #data = {}
+        # return self.baseSender('POST', 'SendSingleRed', data, timeout, **kwargs)
+
+    def send_qzone_red_bag(self) -> dict:
+        '''发送QQ空间红包'''
+        #data = {}
+        # return self.baseSender('POST', 'SendQzoneRed', data, timeout, **kwargs)
+
+    def send_transfer(self) -> dict:
+        '''支付转账'''
+        #data = {}
+        # return self.baseSender('POST', 'Transfer', data, timeout, **kwargs)
+
+    def open_red_bag(self, OpenRedBag) -> dict:
+        '''打开红包 传入红包数据结构'''
+    #     return self.baseSender('POST', 'OpenRedBag', OpenRedBag, timeout, **kwargs)
+
     def add_friend(self, userID: int, groupID: int, content='加个好友!', AddFromSource=2004, timeout=20, **kwargs) -> dict:
         """添加好友"""
         data = {
@@ -431,9 +445,53 @@ class Action:
         }
         return self.baseSender('POST', 'AddQQUser', data, timeout, **kwargs)
 
-    def deal_friend(self) -> dict:
+    def get_friend_file(self, FileID: str, timeout=20, **kwargs) -> dict:
+        """获取好友文件下载链接"""
+        funcname = 'OfflineFilleHandleSvr.pb_ftn_CMD_REQ_APPLY_DOWNLOAD-1200'
+        data = {
+            'FileID': FileID
+        }
+        return self.baseSender('POST', funcname, data, timeout, **kwargs)
+
+    def get_group_file(self, groupID: int, FileID: str, timeout=20, **kwargs) -> dict:
+        """获取群文件下载链接"""
+        funcname = 'OidbSvc.0x6d6_2'
+        data = {
+            'FileID': FileID,
+            'GroupID': groupID
+        }
+        return self.baseSender('POST', funcname, data, timeout, **kwargs)
+
+    def set_group_announce(self, groupID: int, Title: str, Text: str, Pinned=0, Type=10, timeout=5, **kwargs) -> dict:
+        """设置群公告"""
+        data = {
+            'GroupID': groupID,  # 发布的群号
+            "Title": Title,  # 公告标题
+            "Text": Text,  # 公告内容
+            "Pinned": Pinned,  # 1为置顶,0为普通公告
+            "Type": Type  # 发布类型(10为使用弹窗公告,20为发送给新成员,其他暂未知)
+        }
+        try:
+            res = requests.post(f'{self.__host}:{self.__port}/v1/Group/Announce?qq={self.qq}', data=data, timeout=timeout, **kwargs)
+            return res.json()
+        except Exception:
+            return {}
+
+    def deal_friend(self, Action: int) -> dict:
         """处理好友请求"""
-        # TODO
+        # --Action 1 忽略 2 同意 3 拒绝
+        # data = {
+        #     'Action':Action
+        # }
+        # return self.baseSender('POST', 'DealFriend', data, timeout, **kwargs)
+
+    def deal_group(self, Action: int) -> dict:
+        '''处理群邀请'''
+        # --Action 14 忽略 1 同意 21 拒绝
+        # data = {
+        #     'Action':Action
+        # }
+        # return self.baseSender('POST', 'AnswerInviteGroup', data, timeout, **kwargs)
 
     def all_shut_up_on(self, groupid, timeout=20, **kwargs) -> dict:
         """开启全员禁言"""
